@@ -30,8 +30,8 @@ class Installation(object):
 				group=None,
 				sudo=sudo,)
 		else:
-			fp.permission.check(permission=700, sudo=sudo)
-			fp.ownership.check(owner=username, group=None, sudo=sudo)
+			fp.permission.set(permission=700, sudo=sudo)
+			fp.ownership.set(owner=username, group=None, sudo=sudo)
 
 		# the ssh config.
 		fp = Formats.FilePath(f"{home}.ssh/config")
@@ -44,8 +44,8 @@ class Installation(object):
 				group=None,
 				sudo=sudo,)
 		else:
-			fp.permission.check(permission=644, sudo=sudo)
-			fp.ownership.check(owner=username, group=None, sudo=sudo)
+			fp.permission.set(permission=644, sudo=sudo)
+			fp.ownership.set(owner=username, group=None, sudo=sudo)
 
 		# the ssh known hosts.
 		fp = Formats.FilePath(f"{home}.ssh/known_hosts")
@@ -58,8 +58,8 @@ class Installation(object):
 				group=None,
 				sudo=sudo,)
 		else:
-			fp.permission.check(permission=644, sudo=sudo)
-			fp.ownership.check(owner=username, group=None, sudo=sudo)
+			fp.permission.set(permission=644, sudo=sudo)
+			fp.ownership.set(owner=username, group=None, sudo=sudo)
 
 		# authorized keys.
 		fp = Formats.FilePath(f"{home}.ssh/authorized_keys")
@@ -72,8 +72,8 @@ class Installation(object):
 				group=None,
 				sudo=sudo,)
 		else:
-			fp.permission.check(permission=600, sudo=sudo)
-			fp.ownership.check(owner=username, group=None, sudo=sudo)
+			fp.permission.set(permission=600, sudo=sudo)
+			fp.ownership.set(owner=username, group=None, sudo=sudo)
 
 		# success.
 		response["success"] = True
@@ -98,8 +98,8 @@ class Installation(object):
 			response["error"] = f"Required ssh configuration file [{fp.path}] for user [{username}] is not installed."
 			return response
 		else:
-			fp.permission.check(permission=700, sudo=sudo)
-			fp.ownership.check(owner=username, group=None, sudo=sudo)
+			fp.permission.set(permission=700, sudo=sudo)
+			fp.ownership.set(owner=username, group=None, sudo=sudo)
 
 		# the ssh config.
 		fp = Formats.FilePath(f"{home}.ssh/config")
@@ -107,8 +107,8 @@ class Installation(object):
 			response["error"] = f"Required ssh configuration file [{fp.path}] for user [{username}] is not installed."
 			return response
 		else:
-			fp.permission.check(permission=644, sudo=sudo)
-			fp.ownership.check(owner=username, group=None, sudo=sudo)
+			fp.permission.set(permission=644, sudo=sudo)
+			fp.ownership.set(owner=username, group=None, sudo=sudo)
 
 		# the ssh known hosts.
 		fp = Formats.FilePath(f"{home}.ssh/known_hosts")
@@ -116,8 +116,8 @@ class Installation(object):
 			response["error"] = f"Required ssh configuration file [{fp.path}] for user [{username}] is not installed."
 			return response
 		else:
-			fp.permission.check(permission=644, sudo=sudo)
-			fp.ownership.check(owner=username, group=None, sudo=sudo)
+			fp.permission.set(permission=644, sudo=sudo)
+			fp.ownership.set(owner=username, group=None, sudo=sudo)
 			
 		# authorized keys.
 		fp = Formats.FilePath(f"{home}.ssh/authorized_keys")
@@ -125,8 +125,8 @@ class Installation(object):
 			response["error"] = f"Required ssh configuration file [{fp.path}] for user [{username}] is not installed."
 			return response
 		else:
-			fp.permission.check(permission=600, sudo=sudo)
-			fp.ownership.check(owner=username, group=None, sudo=sudo)
+			fp.permission.set(permission=600, sudo=sudo)
+			fp.ownership.set(owner=username, group=None, sudo=sudo)
 
 		# success.
 		response["success"] = True
@@ -1202,7 +1202,11 @@ class SSHD(object):
 		# auth keys.
 		configuration += '\nAuthorizedKeysFile {}'.format(".ssh/authorized_keys")
 
+		# banner.
+		configuration += '\nBanner .ssh/banner'
+
 		# per users.
+		configuration += '\nChallengeResponseAuthentication no'
 		for username, info in users.items():
 			configuration += f'\n# User: {username}'
 
@@ -1211,74 +1215,75 @@ class SSHD(object):
 
 			# authentication by password.
 			if info["password_authentication"]:
-				configuration += '\n     PasswordAuthentication yes'
-				configuration += '\n     PermitEmptyPasswords no'
-				configuration += '\n     ChallengeResponseAuthentication no'
+				configuration += '\n    PasswordAuthentication yes'
+				configuration += '\n    PermitEmptyPasswords no'
 			else:
-				configuration += '\n     PasswordAuthentication no'
-				configuration += '\n     PermitEmptyPasswords no'
-				configuration += '\n     ChallengeResponseAuthentication no'
+				configuration += '\n    PasswordAuthentication no'
+				configuration += '\n    PermitEmptyPasswords no'
 
 			# authentication by keys.
 			if info["key_authentication"]:
-				configuration += '\n     PubkeyAuthentication {}'.format('yes')
+				configuration += '\n    PubkeyAuthentication {}'.format('yes')
 			else:
-				configuration += '\n     PubkeyAuthentication {}'.format('no')
+				configuration += '\n    PubkeyAuthentication {}'.format('no')
 
 
 			# chroot directory.
 			if isinstance(info["chroot_directory"], str):
-				configuration += f'\n     ChrootDirectory {info["chroot_directory"]}'
+				configuration += f'\n    ChrootDirectory {info["chroot_directory"]}'
 
 			# root permission.
 			l = "no"
 			if info["root_permissions"] and info["key_authentication"]: l = "prohibit-password"
 			elif info["root_permissions"]: l = "yes"
-			configuration += f'\n     PermitRootLogin {l}'
+			configuration += f'\n    PermitRootLogin {l}'
 
 			# connection options.
-			configuration += f'\n     X11Forwarding {self.__convert_boolean__(info["x11_forwarding"])}'
-			configuration += f'\n     AllowTcpForwarding {self.__convert_boolean__(info["tcp_forwarding"])}'
+			configuration += f'\n    X11Forwarding {self.__convert_boolean__(info["x11_forwarding"])}'
+			configuration += f'\n    AllowTcpForwarding {self.__convert_boolean__(info["tcp_forwarding"])}'
 
 			# default options.
-			configuration += f'\n     PermitTunnel no'
-			configuration += f'\n     AllowStreamLocalForwarding no'
-			configuration += f'\n     GatewayPorts no'
-			configuration += f'\n     PermitTTY yes'
+			configuration += f'\n    PermitTunnel no'
+			configuration += f'\n    AllowStreamLocalForwarding no'
+			configuration += f'\n    GatewayPorts no'
+			configuration += f'\n    PermitTTY yes'
 			
 			# check ip filter.
 			if info["ip_filter"]:
 
 				# match verified ips.
-				configuration += f'\n     Match User {username} Address {self.__sum_list__(info["allowed_ips"])}'
+				configuration += f'\n    Match User {username} Address {self.__sum_list__(info["allowed_ips"])}'
 
 				# check sftp only.
 				if info["sftp_only"]:
-					configuration += '\n          ForceCommand internal-sftp'
+					configuration += '\n        ForceCommand internal-sftp'
 
 				# shell access.
 				else:
-					configuration += '\n          ForceCommand .ssh/utils/original.sh'
+					configuration += '\n        ForceCommand .ssh/utils/original.sh'
 
 				# match unverified ips.
-				configuration += f'\n     Match User {username} Address *,!{self.__sum_list__(info["allowed_ips"])}'
-				configuration += f'\n          ForceCommand .ssh/utils/log.sh "Your ip address is not authorized." "Authorize your ip address to access user [{username}]."'
+				configuration += f'\n    Match User {username} Address *,!{self.__sum_list__(info["allowed_ips"])}'
+				configuration += f'\n        ForceCommand .ssh/utils/log.sh "Your ip address is not authorized." "Authorize your ip address to access user [{username}]."'
 
 			# no ip filter.
 			else:
 
 				# check sftp only.
 				if info["sftp_only"]:
-					configuration += '\n          ForceCommand internal-sftp'
+					configuration += '\n    ForceCommand internal-sftp'
 
 				# shell access.
 				else:
-					configuration += '\n          ForceCommand .ssh/utils/original.sh'
+					configuration += '\n    ForceCommand .ssh/utils/original.sh'
 
 		# match none authorized users.
 		#if '*all*' not in list(users.keys()):
 		configuration += f'\nMatch User *,!{self.__sum_list__(list(users.keys()))}'
-		configuration += f'\n     ForceCommand .ssh/utils/log.sh "You are not authorized to access user [$USER] over ssh."'
+		configuration += '\n    PasswordAuthentication no'
+		configuration += '\n    PermitEmptyPasswords no'
+		configuration += '\n    PubkeyAuthentication no'
+		configuration += f'\n    ForceCommand .ssh/utils/log.sh "You are not authorized to access user [$USER] over ssh."'
 		configuration += "\n"
 
 		# save sshd.
@@ -1290,6 +1295,7 @@ class SSHD(object):
 			file.file_path.copy(fp.path, sudo=True)
 			fp.permission.set(permission=644, sudo=True)
 			fp.ownership.set(owner="root", group=None, sudo=True)
+			os.system("sudo systemctl restart ssh")
 			if not fp.exists(sudo=True):
 				return r3sponse.error_response(f"Failed to save the sshd configuration.")
 
